@@ -7,10 +7,9 @@
 (*  Copyright 2001 Institut National de Recherche en Informatique et   *)
 (*  Automatique.  Distributed only by permission.                      *)
 (*                                                                     *)
-(*  $Id: explode.ml,v 1.6 2001/05/28 17:28:55 maranget Exp $           *)
+(*  $Id: explode.ml,v 1.7 2012-06-05 14:55:39 maranget Exp $           *)
 (***********************************************************************)
 
-open Lexeme
 open Htmltext
 open Tree
 
@@ -45,8 +44,9 @@ let rec tree env t k = match t with
               of_styles now (Node ([s],ts))::k
       end
 | ONode (so,sc,ts) ->
-    ONode
-      (so,sc, List.fold_right (tree env) ts [])::k
+    of_styles env
+      (ONode
+         (so,sc, List.fold_right (tree []) ts []))::k
 
 
 let trees ts =  List.fold_right (tree []) ts []
