@@ -17,6 +17,8 @@ val hot_start : saved -> unit
 val step_anchor : int -> unit
 val get_anchor : int -> int
 val register : int -> string -> string -> unit
-val flush : (string -> unit)  -> string -> string -> unit
-val some : bool ref
+val sub_notes : unit -> unit
+val flush : bool -> (string -> unit)  -> string -> string -> unit
+val end_notes : unit -> unit
+
 
