@@ -1,5 +1,4 @@
 #! /bin/sh -e
-
 . ./config.sh
 
 cpv () {
@@ -48,7 +47,7 @@ install . $LATEXLIBDIR hevea.sty
 install html $LIBDIR/html $HTMLLIB
 install text $LIBDIR/text $TEXTLIB
 install info $LIBDIR/info $INFOLIB
-MAPPINGS=`( cd mappings ; echo *.map )`
+MAPPINGS=`( cd ./mappings  && echo *.map )`
 install mappings $LIBDIR/mappings $MAPPINGS
 installbin $TARGET hevea hacha esponja bibhva
 cpv imagen $BINDIR
\ No newline at end of file
