@@ -9,8 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-let header = "$Id: lexstate.ml,v 1.70 2007/06/06 18:24:19 maranget Exp $"
-
+open Printf
 open Misc
 open Lexing
 open MyStack
@@ -19,20 +18,28 @@ open MyStack
 
 (* Commands nature *)
 type action =
-  | Subst of string
+  | Subst of string list
   | Toks of string list
   | CamlCode of (Lexing.lexbuf -> unit)
 
+let body_to_string = String.concat ""
+
+let pretty_body chan = List.iter(fprintf chan "%s")
 
 let pretty_action acs =
    match acs with
-   | Subst s    -> Printf.fprintf stderr "{%s}" s
+   | Subst s    ->
+       eprintf "{%a}" (fun chan -> List.iter (fprintf chan "\"%s\" ")) s
    | Toks l ->
        List.iter
          (fun s -> Printf.fprintf stderr "{%s}, " s)
          l
    | CamlCode _ -> prerr_string "*code*"
 
+let rec is_empty_list = function
+  | [] -> true
+  | x::xs -> String.length x = 0 && is_empty_list xs
+
 type pat = string list * string list
 
 
@@ -54,7 +61,7 @@ let zero_pat = latex_pat [] 0
 and one_pat  = latex_pat [] 1
 
 (* Environments *)
-type subst = Top | Env of string arg array
+type subst = Top | Env of string list arg array
 and 'a arg = {arg : 'a ; subst : subst }
 
 let mkarg arg subst = {arg=arg ; subst=subst }
@@ -74,7 +81,6 @@ let stack_subst = MyStack.create "stack_subst"
 and stack_alltt = MyStack.create_init "stack_alltt" Not
 
 let get_subst () = !subst
-let set_subst s = subst := s
 let top_subst = Top
 
 
@@ -85,9 +91,9 @@ let pretty_subst = function
       if Array.length args <> 0 then begin
         prerr_endline "Env: " ;
         for i = 0 to Array.length args - 1 do
-          prerr_string "\t``" ;
-          prerr_string args.(i).arg  ;
-          prerr_endline "''"
+          prerr_string "\t'" ;
+          eprintf "%a" pretty_body args.(i).arg  ;
+          prerr_endline "'"
         done
       end
 
@@ -100,7 +106,7 @@ let rec pretty_subst_rec indent = function
         for i = 0 to Array.length args - 1 do
           prerr_string indent ;
           prerr_string ("  #"^string_of_int (i+1)^" ``");
-          prerr_string  args.(i).arg ;
+          pretty_body stderr args.(i).arg ;
           prerr_endline "''" ;
           pretty_subst_rec ("  "^indent) args.(i).subst
         done
@@ -168,7 +174,6 @@ let pretty_lexbuf lb =
 
   
 (* arguments inside macros*)
-type env = string array ref
 type closenv = string array t
 
 
@@ -188,7 +193,7 @@ let plain_of_char = function
   | '\'' -> 10
   | '`' -> 11
   | '-' -> 12
-  | '"' -> 13
+  | '"' -> 13 (* '"' *)
   | c   ->
       raise
         (Fatal ("Internal catcode table error: '"^String.make 1 c^"'"))
@@ -196,9 +201,15 @@ let plain_of_char = function
 and plain = Array.create 14 true
 
 let is_plain c = plain.(plain_of_char c)
-and set_plain c = plain.(plain_of_char c) <- true
-and unset_plain c = plain.(plain_of_char c) <- false
-and plain_back b c = plain.(plain_of_char c) <- b
+and set_plain c =
+(*  if c = '_' then eprintf "set_plain %c\n" c ; *)
+  plain.(plain_of_char c) <- true
+and unset_plain c =
+(*  if c = '_' then eprintf "unset_plain %c\n" c ; *)
+  plain.(plain_of_char c) <- false
+and plain_back b c =
+(*  if c = '_' then eprintf "plain_back %c <- %b\n" c b ; *)
+  plain.(plain_of_char c) <- b
 
 
 let top_level () = match !subst with Top -> true | _ -> false
@@ -224,7 +235,8 @@ let scan_arg lexfun i =
   let arg = args.(i) in
 
   if !verbose > 1 then begin
-    prerr_string ("Subst arg #"^string_of_int (i+1)^" -> ``"^arg.arg^"''")
+    eprintf
+      "Subst arg #%i -> %a\n" i pretty_body arg.arg
   end ;
   let r = lexfun arg in
   r
@@ -423,14 +435,15 @@ let full_save_arg_limits eoferror parg lexfun lexbuf =
 ;;
 
 
-type ok = No of string | Yes of string
+type ok = No of string | Yes of string list
 ;;
 
 let parg {arg=arg} = arg
 and pargs {arg=args} = String.concat ", " args
+and parg_list {arg=xs} = body_to_string xs
 
 and pok = function
-  | {arg=Yes s} -> s
+  | {arg=Yes s} -> String.concat "" s
   | {arg=No s} -> "* default arg: ["^s^"] *"
 
 
@@ -439,9 +452,9 @@ let eof_arg () =
   raise (Error "Eof while looking for argument")
 
 let save_arg lexbuf =
-  let r = full_save_arg eof_arg mkarg parg Save.arg lexbuf in
-  r
-
+  full_save_arg eof_arg mkarg parg Save.arg lexbuf
+and save_body lexbuf =
+    full_save_arg eof_arg mkarg parg_list Save.arg_list lexbuf
 and save_arg_with_delim delim lexbuf =
   full_save_arg eof_arg mkarg parg (Save.with_delim delim) lexbuf
 and save_filename lexbuf =
@@ -452,7 +465,7 @@ and save_xy_arg lexbuf =
   full_save_arg eof_arg mkarg parg Save.xy_arg lexbuf
 and save_cite_arg lexbuf =
   full_save_arg eof_arg mkarg pargs Save.cite_arg lexbuf
-  
+
 type sup_sub = {
   limits : Misc.limits option ;
   sup : string arg ;
@@ -531,7 +544,7 @@ let save_arg_opt default lexbuf =
       mkarg
       pok
       (fun lexbuf ->
-        try Yes (Save.opt lexbuf) with          
+        try Yes (Save.opt_list lexbuf) with          
         | Save.NoOpt -> No default)
       lexbuf in
   match r.arg with
@@ -548,10 +561,10 @@ let from_ok okarg = match okarg.arg with
       mkarg s okarg.subst
   | No s  ->
       optarg := false ;
-      mkarg s okarg.subst
+      mkarg [s] okarg.subst
 
 let pretty_ok = function
-  Yes s -> "+"^s^"+"
+  Yes s -> "+"^String.concat "" s^"+"
 | No s  -> "-"^s^"-"
 ;;
 
@@ -560,20 +573,22 @@ let norm_arg s =
   String.length s = 2 && s.[0] = '#' &&
   ('0' <= s.[1] && s.[1] <= '9')
 
+let list_arg a = { a with arg = [a.arg] }
+
 let rec parse_args_norm pat lexbuf = match pat with
 |   [] -> []
 | s :: (ss :: _ as pat) when norm_arg s && norm_arg ss ->
-    let arg = save_arg lexbuf in
+    let arg = save_body lexbuf in
     let r = parse_args_norm pat lexbuf in
      arg :: r
 | s :: ss :: pat when norm_arg s && not (norm_arg ss) ->
     let arg = save_arg_with_delim ss lexbuf in
-    arg :: parse_args_norm pat lexbuf
+    list_arg arg :: parse_args_norm pat lexbuf
 | s :: pat when not (norm_arg s) ->
     Save.skip_delim s lexbuf ;
     parse_args_norm pat lexbuf
 | _ :: pat ->
-    let arg = save_arg lexbuf in
+    let arg = save_body lexbuf in
     let r = parse_args_norm pat lexbuf in
     arg :: r
 ;;
@@ -588,7 +603,7 @@ let skip_opt lexbuf =
   let _ =  save_arg_opt "" lexbuf  in
   ()
 
-and save_opt def lexbuf = from_ok (save_arg_opt def  lexbuf)
+and save_opt def lexbuf = from_ok (save_arg_opt def lexbuf)
 ;;
 
 let rec save_opts pat lexbuf = match pat with
@@ -621,7 +636,7 @@ let make_stack name pat lexbuf =
       pretty_pat pat ;
       prerr_endline "";
       for i = 0 to Array.length args-1 do
-        Printf.fprintf stderr "\t#%d = %s\n" (i+1) (args.(i).arg) ;
+        Printf.fprintf stderr "\t#%d = %a\n" (i+1) pretty_body args.(i).arg ;
         pretty_subst (args.(i).subst)
       done
     end ;
@@ -640,7 +655,7 @@ let scan_this lexfun s =
     Printf.fprintf stderr "scan_this : [%s]" s ;
     prerr_endline ""  
   end ;
-  let lexbuf = Lexing.from_string s in
+  let lexbuf = MyLexing.from_string s in
   let r = lexfun lexbuf in
   if !verbose > 1 then begin
     Printf.fprintf stderr "scan_this : over" ;
@@ -649,6 +664,21 @@ let scan_this lexfun s =
   restore_lexstate ();
   r
 
+and scan_this_list lexfun xs =
+  start_lexstate ();
+  if !verbose > 1 then begin
+    eprintf "scan_this_list : [%a]" pretty_body xs ;
+    prerr_endline ""  
+  end ;
+  let lexbuf = MyLexing.from_list xs in
+  let r = lexfun lexbuf in
+  if !verbose > 1 then begin
+    Printf.fprintf stderr "scan_this_list : over" ;
+    prerr_endline ""
+  end ;
+  restore_lexstate ();
+  r
+
 and scan_this_arg lexfun {arg=s ; subst=this_subst } =
   start_lexstate () ;
   subst := this_subst ;
@@ -656,7 +686,7 @@ and scan_this_arg lexfun {arg=s ; subst=this_subst } =
     Printf.fprintf stderr "scan_this_arg : [%s]" s ;
     prerr_endline ""  
   end ;
-  let lexbuf = Lexing.from_string s in
+  let lexbuf = MyLexing.from_string s in
   let r = lexfun lexbuf in
   if !verbose > 1 then begin
     Printf.fprintf stderr "scan_this_arg : over" ;
@@ -664,6 +694,22 @@ and scan_this_arg lexfun {arg=s ; subst=this_subst } =
   end ;
   restore_lexstate ();
   r
+
+and scan_this_arg_list lexfun {arg=xs ; subst=this_subst } =
+  start_lexstate () ;
+  subst := this_subst ;
+  if !verbose > 1 then begin
+    Printf.fprintf stderr "scan_this_arg_list : [%a]\n%!"
+      pretty_body xs
+  end ;
+  let lexbuf = MyLexing.from_list xs in
+  let r = lexfun lexbuf in
+  if !verbose > 1 then begin
+    Printf.fprintf stderr "scan_this_arg : over\n%!"
+  end ;
+  restore_lexstate ();
+  r
+
 ;;
 
 let scan_this_may_cont lexfun lexbuf cur_subst
@@ -680,7 +726,7 @@ let scan_this_may_cont lexfun lexbuf cur_subst
   save_lexstate ();
   record_lexbuf lexbuf cur_subst ;
   subst := env ;
-  let lexer = Lexing.from_string s in
+  let lexer = MyLexing.from_string s in
   let r = lexfun lexer in
 
   restore_lexstate ();
@@ -690,6 +736,29 @@ let scan_this_may_cont lexfun lexbuf cur_subst
   end ;
   r
 
+let scan_this_list_may_cont lexfun lexbuf cur_subst
+    {arg=s ; subst=env } =
+  if !verbose > 1 then begin
+    eprintf "scan_this_list_may_cont : [%a]\n%!" pretty_body s ;
+    if !verbose > 1 then begin
+      prerr_endline "Pushing lexbuf and env" ;
+      pretty_lexbuf lexbuf ;
+      pretty_subst !subst
+    end
+  end ;
+  save_lexstate ();
+  record_lexbuf lexbuf cur_subst ;
+  subst := env ;
+  let lexer = MyLexing.from_list s in
+  let r = lexfun lexer in
+
+  restore_lexstate ();
+  if !verbose > 1 then begin
+    Printf.fprintf stderr "scan_this_list_may_cont : over" ;
+    prerr_endline ""
+  end ;
+  r
+
 let real_input_file loc_verb main filename input =
   if !verbose > 0 then
     prerr_endline ("Input file: "^filename) ;
