@@ -1078,8 +1078,8 @@ register_init "german"
   (fun () ->
       def_code "\\@german@dquote"
         (fun lexbuf ->
-          if effective !alltt then
-            Dest.put_char '"'
+          if effective !alltt || not (is_plain '"') then (* '"' *)
+            Dest.put_char '"' (* '"' *)
           else try
             let c = Save.peek_next_char lexbuf  in
             match c with
