@@ -7,9 +7,10 @@
 (*  Copyright 2001 Institut National de Recherche en Informatique et   *)
 (*  Automatique.  Distributed only by permission.                      *)
 (*                                                                     *)
-(*  $Id: ultra.ml,v 1.13 2006/10/09 08:25:16 maranget Exp $             *)
+(*  $Id: ultra.ml,v 1.14 2012-06-05 14:55:39 maranget Exp $             *)
 (***********************************************************************)
 
+open Printf
 open Tree
 open Htmltext
 open Util
@@ -95,13 +96,16 @@ let get_sames ts fs =
           rem in
   do_rec [] fs
 
-
-         
 let group_font ts fs =
   let fonts,no_fonts =
     List.partition (fun (_,f) -> is_font f.nat) fs in
   get_sames ts fonts@no_fonts
 
+let group_span ts fs =
+  let span,no_span =
+    List.partition (fun (_,f) -> is_span f.nat) fs in
+  get_sames ts span@no_span
+                      
 let conflict_low i1 j1 i2 j2 =  i1 < i2 && i2 <= j1 && j1 < j2
 
 let correct_cfl_low ts i1 j1 i2 j2 =
@@ -200,6 +204,7 @@ let factorize low high ts =
       r in
   let r = do_rec low [] [] in
   let r = group_font ts r in
+  let r = group_span ts r in
   let r = extend_neutrals ts r in
   if r <> [] && !Emisc.verbose > 1 then begin
     Printf.fprintf stderr "Factors in %d %d\n" low high ;
@@ -346,51 +351,6 @@ let  clean t k = match t with
   | Node ([],ts) -> ts@k
   | _ -> t::k
 
-let rec as_long p = function
-  | x::rem when p x ->
-      let yes,no = as_long p rem in
-      x::yes,no
-  | l -> [],l
-
-let rec as_long_end p = function
-  | [] -> [],[]
-  | x::rem ->
-      match as_long_end p rem with
-      | [],no when p x -> [],x::no
-      | yes,no -> x::yes,no
-          
-
-      
-let bouts _p ts =
-  let bef,rem = as_long is_blank ts in
-  let inside,aft = as_long_end is_blank rem in
-  bef,inside,aft
-
-exception Failed
-
-let extract_props_trees ps ts =
-  let card = List.length ps in
-  let rec do_rec seen = function
-    | [] -> seen,[]
-    | Blanks _ as t::rem ->
-        begin match do_rec seen rem with
-        | r,rem -> r,t::rem
-        end
-    | Node (s,args)::rem ->
-        let lift,keep = extract_props ps s in
-        let seen = union seen lift in
-        if List.length seen > card then
-          raise Failed
-        else
-          let r,rem = do_rec seen rem in
-          begin match keep with
-          | [] -> r,args@rem
-          | _  -> r,Node (keep,args)::rem
-          end
-    | _ -> assert false in
-  do_rec [] ts
-
-
 let rec neutrals started r = function
   | [] -> r
   | Blanks _::rem -> neutrals started r rem
@@ -437,10 +397,6 @@ let remove s = function
   | t -> t
 
 
-let is_text = function
-  | Text _ -> true
-  | _ -> false
-
 and is_text_blank = function
   | Text _ | Blanks _ -> true
   | _ -> false
@@ -524,7 +480,7 @@ and trees i j ts k =
 
 and opt_onodes ts i = match ts.(i) with
   |  ONode (o,c,args) -> begin match opt false (Array.of_list args) [] with
-      | [Node (s,args)] ->
+      | [Node (s,args)] when false ->
 	  let s1, s2 = partition_color s in
 	  ts.(i) <-
 	     begin match s1, s2 with
@@ -569,6 +525,12 @@ and extend_blanks pre ts post k = match ts with
 
 
 let main chan ts =
+  if !Emisc.verbose > 2 then begin
+    eprintf "**Ultra input **\n" ;
+    Pp.ptrees stderr ts ;
+    eprintf "** Ultra end**\n%!" ;
+    ()
+  end ;
   let ci = costs Htmllex.cost ts in
   let rs =  opt true (Array.of_list (Explode.trees ts)) [] in
   let cf = costs Htmltext.cost rs in
@@ -580,6 +542,12 @@ let main chan ts =
       Pp.trees stderr rs
     end ;
     Pp.ptrees chan ts
-  end else
+  end else begin
+    if !Emisc.verbose > 2 then begin
+      eprintf "** Ultra output **\n" ;
+      Pp.trees stderr rs ;
+      eprintf "** Ultra end**\n%!" ;
+      ()
+    end ;
     Pp.trees chan rs 
-
+  end
